/* Schleifenbauer data base tables and triggers, DM v1.20

        Updates v1.32 to v1.40

        adds support for extended channel info; outlets up to 48
        added support for environment sensors
	increased fixed decimal to 2 decimal digits
*/


/* adjust data base name ! */
USE `sppdu`;


INSERT INTO dbhistory(version) VALUES("1.40");

ALTER TABLE `splog` CHANGE COLUMN `value` `value` DECIMAL(9,2) NULL DEFAULT NULL  ;


ALTER TABLE `spdm` ADD COLUMN `extended` int(2) default 0;
ALTER TABLE `spregister` ADD COLUMN `extended` int(2) default 0;
ALTER TABLE `splog` ADD COLUMN `extended` int(2) default 0;
ALTER TABLE `splog` ADD INDEX `extended` (`extended` ASC) ;

update spdm set extended=1 where mnemonic="rsomks";
update spdm set extended=1 where mnemonic="stomcm"; 
update spdm set extended=1 where mnemonic="stolgp"; 
update spdm set extended=1 where mnemonic="stolnm"; 
update spdm set extended=1 where mnemonic="stiodl"; 
update spdm set extended=1 where mnemonic="swocst"; 
update spdm set extended=1 where mnemonic="swosch";  
update spdm set extended=1 where mnemonic="swounl"; 
update spdm set extended=1 where mnemonic="omkwht"; 
update spdm set extended=1 where mnemonic="omkwhs"; 
update spdm set extended=1 where mnemonic="ompfac"; 
update spdm set extended=1 where mnemonic="omcrac"; 
update spdm set extended=1 where mnemonic="omcrpk"; 
update spdm set extended=1 where mnemonic="omvoac"; 

INSERT INTO `spdm` (`interval`, `mnemonic`, `name`, `register`, `description`,`group` ) VALUES (0,  'cfnres', 'n.o.Env.Sensor', 206, 'Returns the number of detected environmental sensors on the sensor port.', 'configuration' );
INSERT INTO `spdm` (`interval`, `mnemonic`, `name`, `register`, `description`,`group` ) VALUES (60, 'snstyp', 'sensorType', 5008, 'Returns the detected sensor type, can be T: temperature, H: Humidity, I: dry switch input, O switch output.', 'pdu_measures');
INSERT INTO `spdm` (`interval`, `mnemonic`, `name`, `register`, `description`,`group` ) VALUES (60, 'snsval', 'sensorValue', 5024, 'Returns the sensor value, can be for T: temperature in degree Celsius, for H: Humidity in percent, for both I: and O switch state as 0 or 1 or pulse time in secs.', 'pdu_measures' );

DELIMITER $$$


DROP PROCEDURE IF EXISTS `sp_insert_val`$$$
CREATE PROCEDURE `sp_insert_val`(insint INT, itime DATETIME, a INT, raw blob, x int, t tinyblob, b int, l int, n int, r tinyblob)
DETERMINISTIC
BEGIN
  DECLARE i INT;
  DECLARE pos INT;
  DECLARE cnt INT;
  DECLARE intv_cnt INT;
  DECLARE chvalue FLOAT; 
  DECLARE bString tinyblob;
  DECLARE tString VARCHAR(20);
  DECLARE extn INT;
  

  SELECT COUNT(`spdm`.`interval`) FROM `spdm` WHERE
	`mnemonic` = r
    INTO intv_cnt;

   IF (intv_cnt = 1) THEN
     SELECT `spdm`.`interval` FROM `spdm` WHERE
	`mnemonic` = r 
     INTO insint;
   END IF;

  SET cnt = 0;

/* only add records if insint more than 0 seconds */
if (insint = 0) THEN
  DELETE FROM `splog` WHERE `pdu` = a AND `register` = r AND `extended` = x ;
ELSE
/*
	if there are any recent entries in splog,
	thus from last insint seconds, then do not insert new records
*/
  SELECT COUNT(`splog`.`id`) FROM `splog` WHERE
	`pdu` = a AND
	`register` = r AND
    `extended` = x AND
	(TIMESTAMPDIFF (SECOND,`time`,itime) < insint)
    INTO cnt;

END IF;

if (cnt = 0) THEN
SET extn = n;
IF (x = 1) THEN
 SET extn = n-27;
ELSE
  IF (n>27) THEN
    SET extn=27;
  END IF;
END IF;

  SET i = 0;
      WHILE i < extn do

      CASE

        WHEN t like "fd" THEN
		BEGIN
			SET chvalue = sp_get_fd(substr(raw, (b + (i * l))+1,l));
          	INSERT INTO `splog` (`pdu`,`register`,`channel`,`value`,`time`,`min_v`,`max_v`,`extended`)
            	VALUES (a,r,(x * 27)+i+1, chvalue, itime, chvalue, chvalue,x);
		END;

        WHEN t like "int" THEN
		BEGIN
			SET chvalue = sp_get_int(substr(raw, (b + (i * l))+1,l),l);
         	INSERT INTO `splog` (`pdu`,`register`,`channel`,`value`,`time`,`min_v`,`max_v`,`extended`)
            	VALUES (a,r, (x * 27)+i+1, chvalue, itime, chvalue, chvalue,x);
		END;

        WHEN t like "ascii" THEN
		SET bString = substr(raw, (b + (i * l))+1,l);    /* get string in blob */
		SET pos = position("\0" IN bString);             /* find NUL terminator */

		IF (pos = 0) THEN
			SET pos = l+1;
		END IF;

		SET tString = substr(bString, 1, pos-1);         /* convert to C string */

         INSERT INTO `splog` (`pdu`,`register`,`channel`,`text`,`time`,`extended`)
            VALUES (a,r,(x * 27)+i+1, tString, itime,x);

        END CASE;

          SET i = i + 1;
      END WHILE;
	END IF;


END $$$


DROP function IF EXISTS `sp_get_fd`$$$
CREATE FUNCTION `sp_get_fd`(strval tinyblob) RETURNS decimal(9,2)
    DETERMINISTIC
BEGIN
DECLARE sp_get_fd DECIMAL(9,2) DEFAULT 0;
  SET sp_get_fd = sp_get_int(strval,2)/100; 
  RETURN sp_get_fd;
END$$$



DROP TRIGGER IF EXISTS `sp_process_raw`$$$
CREATE TRIGGER `sp_process_raw`
BEFORE INSERT on `spregister`
FOR EACH ROW
BEGIN
DECLARE reg INT;
DECLARE cfnrph INT;
DECLARE cfnrno INT;
DECLARE cfnrso INT;
DECLARE cfnrmo INT;
DECLARE cfamps INT;
DECLARE cfnrte INT;
DECLARE cfnres INT;
SET reg = new.register;
SET new.inserttime=CURRENT_TIMESTAMP;


/* the first parameter is the minimal delay between two inserts.
0 will prevent insertion of records and instead update records,

use a zero setting for static data (configuration) and non zero for dynamic data (measurements)
*/

/*
	first check config records to see how many channels are configured for this register
	limit the number of inserted record to number of channels or even zero

	1.40 add extended attribute
*/

CASE reg
WHEN 100  THEN
BEGIN
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 2, 1, 'idspdm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 2, 2, 1, 'idfwvs');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 4, 16, 1, 'idonbr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 20, 16, 1, 'idpart');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 36, 16, 1, 'idsnbr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 52, 2, 3, 'idchip');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 58, 2, 1, 'idaddr');
END;
WHEN 200  THEN
BEGIN
-- note: cfnr** must be static (interval 0)!
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 1, 1, 'cfnrph');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 1, 1, 1, 'cfnrno');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 2, 1, 1, 'cfnrso');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 3, 1, 1, 'cfnrmo');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 4, 1, 1, 'cfamps');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 5, 1, 1, 'cfnrte');
                    CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 6, 1, 1, 'cfnres');
END; 
WHEN 300  THEN
BEGIN
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 1, 1, 'ssstat');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 1, 1, 1, 'ssttri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 2, 1, 1, 'ssitri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 3, 1, 1, 'ssotri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 4, 1, 1, 'ssvtri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 5, 1, 1, 'ssftri');
  CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 6, 1, 1, 'ssicda');
END;
WHEN 400  THEN 
BEGIN
END; 
WHEN 1000  THEN 
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrph' INTO cfnrph;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrmo' INTO cfnrmo;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrno' INTO cfnrno;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrso' INTO cfnrso;
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 0, 16, 1, 'stdvnm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 16, 16, 1, 'stdvlc');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 32, 20, 1, 'stuser');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 52, 2, 1, 'stpkdr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 54, 2, 1, 'stdpdr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 56, 2, 1, 'stfodl');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 58, 1, 1, 'stpsav');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 59, 1, 1, 'stopom');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 60, 1, 1, 'stmaxt');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 61, 1, 1, 'stdiso');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'fd', 62, 2, cfnrph, 'stimcm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'fd', 68, 2, cfnrmo, 'stomcm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 122, 2, cfnrno, 'stolgp');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 176, 8, cfnrno, 'stolnm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 392, 2, cfnrso, 'stiodl');
  CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 446, 12, 1, 'stcddt');
END;
WHEN 2000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrso' INTO cfnrso;
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 1, cfnrso, 'swocst');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 27, 1, cfnrso, 'swosch');
END;
WHEN 3000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrph' INTO cfnrph;
	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int', 0, 3, cfnrph, 'imkwht');
	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int', 9, 3, cfnrph, 'imkwhs');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 18, 2, cfnrph, 'impfac');
	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 24, 2, cfnrph, 'imcrac');
	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 30, 2, cfnrph, 'imcrpk');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 36, 2, cfnrph, 'imvoac');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 42, 2, cfnrph, 'imvodp');
END;
WHEN 4000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrmo' INTO cfnrmo;
 	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int',  0, 3, cfnrmo, 'omkwht');
	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int', 81, 3, cfnrmo, 'omkwhs');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 162, 2, cfnrmo, 'ompfac');
 	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 216, 2, cfnrmo, 'omcrac');
	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 270, 2, cfnrmo, 'omcrpk');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 324, 2, cfnrmo, 'omvoac');
END;
WHEN 5000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrte' INTO cfnrte;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnres' INTO cfnres;
	IF (cfnrte>0) THEN
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 0, 2, 1, 'pditem');
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 4, 2, 1, 'pdinpk');
	END IF;
	IF (cfnrte>1) THEN
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 2, 2, 1, 'pdetem');
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 6, 2, 1, 'pdexpk');
	END IF;
	IF (cfnres>1) THEN
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'ascii', 8, 1, cfnres, 'snstyp');
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd',   24, 2, cfnres, 'snsval');
	END IF;
END;
 ELSE
  BEGIN
  END;

END CASE;


SET new.done = 1;

END $$$

